@@ -1,63 +1,67 @@
 # Node models to represent channel's tree
 
-import uuid
 import json
-import sys
-from le_utils.constants import content_kinds,file_formats, format_presets, licenses, exercises
-from ..exceptions import InvalidNodeException, InvalidFormatException
-from .. import config
+import uuid
 
-def guess_content_kind(files, questions=None):
-    """ guess_content_kind: determines what kind the content is
-        Args:
-            files (str or list): files associated with content
-        Returns: string indicating node's kind
-    """
-    # Get files and questions into readable format
-    files = [files] if isinstance(files, str) else files
-    questions = [questions] if isinstance(questions, str) else questions
+from le_utils.constants import content_kinds, exercises, file_formats, format_presets, languages
 
-    # If there are any questions, return exercise
-    if questions is not None and len(questions) > 0:
-        return content_kinds.EXERCISE
+from ricecooker.classes.files import NodeFile
+from .licenses import License
+from .. import config, __version__
+from ..exceptions import InvalidNodeException
 
-    # See if any files match a content kind
-    elif files is not None and len(files) > 0:
-        for f in files:
-            ext = f.rsplit('/', 1)[-1].split(".")[-1].lower()
-            if ext in content_kinds.MAPPING:
-                return content_kinds.MAPPING[ext]
-        raise InvalidFormatException("Invalid file type: Allowed formats are {0}".format([key for key, value in content_kinds.MAPPING.items()]))
-
-    # If there are no files/questions, return topic
-    else:
-        return content_kinds.TOPIC
+MASTERY_MODELS = [id for id, name in exercises.MASTERY_MODELS]
 
 
 class Node(object):
     """ Node: model to represent all nodes in the tree """
-    def __init__(self, title, description=None, thumbnail=None, license=None, copyright_holder=None, files=None):
-        self.children = []
+    license = None
+    language = None
+
+    def __init__(self, title, language=None, description=None, thumbnail=None, files=None):
         self.files = []
+        self.children = []
+        self.descendants = []
         self.parent = None
         self.node_id = None
         self.content_id = None
         self.title = title
+        self.hashed_file_name = None
+        self.set_language(language)
         self.description = description or ""
-        self.license = license
-        self.copyright_holder = copyright_holder
 
         for f in files or []:
             self.add_file(f)
 
-        if thumbnail and isinstance(thumbnail, str):
-            from .files import ThumbnailFile
-            self.thumbnail = ThumbnailFile(path=thumbnail)
-            self.add_file(self.thumbnail)
+        self.set_thumbnail(thumbnail)
 
+    def set_language(self, language):
+        """ Set self.language to internal lang. repr. code from str or Language object. """
+        if isinstance(language, str):
+            language_obj = languages.getlang(language)
+            if language_obj:
+                self.language = language_obj.code
+            else:
+                raise TypeError("Language code {} not found".format(language))
+        if isinstance(language, languages.Language):
+            self.language = language.code
 
     def __str__(self):
-        pass
+        count = self.count()
+        metadata = "{0} {1}".format(count, "descendant" if count == 1 else "descendants")
+        return "{title} ({kind}): {metadata}".format(title=self.title, kind=self.__class__.__name__, metadata=metadata)
+
+    def truncate_fields(self):
+        if len(self.title) > config.MAX_TITLE_LENGTH:
+            config.print_truncate("title", self.source_id, self.title, kind=self.kind)
+            self.title = self.title[:config.MAX_TITLE_LENGTH]
+
+        if self.source_id and len(self.source_id) > config.MAX_SOURCE_ID_LENGTH:
+            config.print_truncate("source_id", self.source_id, self.source_id, kind=self.kind)
+            self.source_id = self.source_id[:config.MAX_SOURCE_ID_LENGTH]
+
+        for f in self.files:
+            f.truncate_fields()
 
     def to_dict(self):
         """ to_dict: puts data in format CC expects
@@ -80,18 +84,71 @@ class Node(object):
             Args: file_to_add (File): file model to add to node
             Returns: None
         """
+        from .files import File
+        assert isinstance(file_to_add, File), "Files being added must be instances of a subclass of File class"
         file_to_add.node = self
-        self.files.append(file_to_add)
+        if file_to_add not in self.files:
+            self.files.append(file_to_add)
 
-    def process_files(self):
-        """ process_files: Process node's files
-            Args: None
+    def derive_thumbnail(self):
+        pass
+
+    def has_thumbnail(self):
+        from .files import ThumbnailFile
+        return any(f for f in self.files if isinstance(f, ThumbnailFile))
+
+    def set_thumbnail(self, thumbnail):
+        """ set_thumbnail: Set node's thumbnail
+            Args: thumbnail (ThumbnailFile): file model to add to node
             Returns: None
         """
-        filenames = []
+        self.thumbnail = thumbnail
+        if isinstance(self.thumbnail, str):
+            from .files import ThumbnailFile
+            self.thumbnail = ThumbnailFile(path=self.thumbnail)
+
+        if self.thumbnail:
+            self.add_file(self.thumbnail)
+
+    def get_thumbnail_preset(self):
+        """
+        Returns the format preset corresponding to this Node's type, or None if the node doesn't have a format preset.
+        """
+        if isinstance(self, ChannelNode):
+            return format_presets.CHANNEL_THUMBNAIL
+        elif isinstance(self, TopicNode):
+            return format_presets.TOPIC_THUMBNAIL
+        elif isinstance(self, VideoNode):
+            return format_presets.VIDEO_THUMBNAIL
+        elif isinstance(self, AudioNode):
+            return format_presets.AUDIO_THUMBNAIL
+        elif isinstance(self, DocumentNode):
+            return format_presets.DOCUMENT_THUMBNAIL
+        elif isinstance(self, ExerciseNode):
+            return format_presets.EXERCISE_THUMBNAIL
+        elif isinstance(self, HTML5AppNode):
+            return format_presets.HTML5_THUMBNAIL
+        else:
+            return None
+
+    def process_files(self):
+        """
+        Processes all the files associated with this Node. Files are downloaded if not present in the local storage.
+        Creates and processes a NodeFile containing this Node's metadata.
+        :return: A list of names of all the processed files.
+        """
+        file_names = []
         for f in self.files:
-            filenames.append(f.process_file())
-        return filenames
+            file_names.append(f.process_file())
+
+        if not self.has_thumbnail() and config.THUMBNAILS:
+            file_names.append(self.derive_thumbnail())
+
+        node_file = NodeFile(self.to_dict())
+        self.hashed_file_name = node_file.process_file()
+        file_names.append(self.hashed_file_name)
+
+        return file_names
 
     def count(self):
         """ count: get number of nodes in tree
@@ -103,6 +160,15 @@ class Node(object):
             total += child.count()
         return total
 
+    def get_non_topic_descendants(self):
+        if len(self.descendants) == 0:
+            for child_node in self.children:
+                if child_node.kind == content_kinds.TOPIC:
+                    self.descendants += child_node.get_non_topic_descendants()
+                elif child_node not in self.descendants:
+                    self.descendants.append(child_node)
+        return self.descendants
+
     def print_tree(self, indent=2):
         """ print_tree: prints out structure of tree
             Args: indent (int): What level of indentation at which to start printing
@@ -150,8 +216,6 @@ class ChannelNode(Node):
             title (str): name of channel
             description (str): description of the channel (optional)
             thumbnail (str): file path or url of channel's thumbnail (optional)
-            license (str): default license to use if nodes don't have license specified (optional)
-            copyright_holder (str): name of person or organization who owns license
             files ([<File>]): list of file objects for node (optional)
     """
     kind = "Channel"
@@ -171,10 +235,11 @@ class ChannelNode(Node):
     def get_node_id(self):
         return uuid.uuid5(self.get_domain_namespace(), self.source_id)
 
-    def __str__(self):
-        count = self.count()
-        metadata = "{0} {1}".format(count, "descendant" if count == 1 else "descendants")
-        return "{title} ({kind}): {metadata}".format(title=self.title, kind=self.__class__.__name__, metadata=metadata)
+    def truncate_fields(self):
+        if self.description and len(self.description) > config.MAX_DESCRIPTION_LENGTH:
+            config.print_truncate("description", self.source_id, self.description, kind=self.kind)
+            self.description = self.description[:config.MAX_DESCRIPTION_LENGTH]
+        super(ChannelNode, self).truncate_fields()
 
     def to_dict(self):
         """ to_dict: puts data in format CC expects
@@ -185,9 +250,12 @@ class ChannelNode(Node):
             "id": self.get_node_id().hex,
             "name": self.title,
             "thumbnail": self.thumbnail.filename if self.thumbnail else None,
+            "language" : self.language,
             "description": self.description or "",
             "license": self.license,
-            "copyright_holder": self.copyright_holder or "",
+            "source_domain": self.source_domain,
+            "source_id": self.source_id,
+            "ricecooker_version": __version__,
         }
 
     def validate(self):
@@ -197,12 +265,13 @@ class ChannelNode(Node):
         """
         try:
             assert isinstance(self.source_domain, str), "Channel domain must be a string"
+            assert self.language, "Channel must have a language"
             return super(ChannelNode, self).validate()
         except AssertionError as ae:
             raise InvalidNodeException("Invalid channel ({}): {} - {}".format(ae.args[0], self.title, self.__dict__))
 
 
-class ContentNode(Node):
+class TreeNode(Node):
     """ Model representing the content nodes in the channel's tree
 
         Base model for different content node kinds (topic, video, exercise, etc.)
@@ -210,11 +279,10 @@ class ContentNode(Node):
         Attributes:
             source_id (str): content's original id
             title (str): content's title
+            license (str or <License>): content's license
             description (str): description of content (optional)
             author (str): who created the content (optional)
-            license (str): content's license based on le_utils.constants.licenses
             thumbnail (str): local path or url to thumbnail image (optional)
-            copyright_holder (str): name of person or organization who owns license (optional)
             files ([<File>]): list of file objects for node (optional)
             extra_fields (dict): any additional data needed for node (optional)
             domain_ns (str): who is providing the content (e.g. learningequality.org) (optional)
@@ -224,16 +292,11 @@ class ContentNode(Node):
         assert isinstance(source_id, str), "source_id must be a string"
         self.source_id = source_id
         self.author = author or ""
-        self.license = license
         self.domain_ns = domain_ns
         self.questions = self.questions if hasattr(self, 'questions') else [] # Needed for to_dict method
         self.extra_fields = extra_fields or {}
-        super(ContentNode, self).__init__(title, **kwargs)
 
-    def __str__(self):
-        count = self.count()
-        metadata = "{0} {1}".format(count, "descendant" if count == 1 else "descendants")
-        return "{title} ({kind}): {metadata}".format(title=self.title, kind=self.__class__.__name__, metadata=metadata)
+        super(TreeNode, self).__init__(title, **kwargs)
 
     def get_domain_namespace(self):
         if not self.domain_ns:
@@ -251,23 +314,38 @@ class ContentNode(Node):
             self.node_id = uuid.uuid5(self.parent.get_node_id(), self.get_content_id().hex)
         return self.node_id
 
+
+    def truncate_fields(self):
+        if self.author and len(self.author) > config.MAX_AUTHOR_LENGTH:
+            config.print_truncate("author", self.source_id, self.author, kind=self.kind)
+            self.author = self.author[:config.MAX_AUTHOR_LENGTH]
+
+        self.license and self.license.truncate_fields()
+
+        super(TreeNode, self).truncate_fields()
+
+
     def to_dict(self):
         """ to_dict: puts data in format CC expects
             Args: None
-            Returns: dict of node's data
+            Returns: dict of channel data
         """
         return {
             "title": self.title,
+            "language" : self.language,
             "description": self.description,
             "node_id": self.get_node_id().hex,
             "content_id": self.get_content_id().hex,
+            "source_domain": self.domain_ns.hex,
+            "source_id": self.source_id,
             "author": self.author,
-            "files" : [f.to_dict() for f in filter(lambda x: x and x.filename, self.files)], # Filter out failed downloads
+            "files" : [f.to_dict() for f in self.files if f and f.filename], # Filter out failed downloads
             "kind": self.kind,
-            "license": self.license,
-            "copyright_holder": self.copyright_holder or "",
-            "questions": [question.to_dict() for question in self.questions],
-            "extra_fields": json.dumps(self.extra_fields),
+            "license": None,
+            "license_description": None,
+            "copyright_holder": "",
+            "questions": [],
+            "extra_fields": {},
         }
 
     def validate(self):
@@ -276,14 +354,13 @@ class ContentNode(Node):
             Returns: boolean indicating if content node is valid
         """
         assert isinstance(self.author, str) , "Assumption Failed: Author is not a string"
-        assert isinstance(self.license, str) or self.license is None, "Assumption Failed: License is not a string or empty"
         assert isinstance(self.files, list), "Assumption Failed: Files is not a list"
         assert isinstance(self.questions, list), "Assumption Failed: Questions is not a list"
         assert isinstance(self.extra_fields, dict), "Assumption Failed: Extra fields is not a dict"
-        return super(ContentNode, self).validate()
+        return super(TreeNode, self).validate()
 
 
-class TopicNode(ContentNode):
+class TopicNode(TreeNode):
     """ Model representing channel topics
 
         Topic nodes are used to add organization to the channel's content
@@ -292,22 +369,13 @@ class TopicNode(ContentNode):
             source_id (str): content's original id
             title (str): content's title
             description (str): description of content (optional)
-            author (str): who created the content (optional)
-            license (str): default license for content under this topic
             thumbnail (str): local path or url to thumbnail image (optional)
-            copyright_holder (str): name of person or organization who owns license (optional)
-            extra_fields (dict): any additional data needed for node (optional)
-            domain_ns (str): who is providing the content (e.g. learningequality.org) (optional)
     """
-
-    def __init__(self, *args, **kwargs):
-        self.kind = content_kinds.TOPIC
-        super(TopicNode, self).__init__(*args, **kwargs)
-
-    def __str__(self):
-        count = self.count()
-        metadata = "{0} {1}".format(count, "descendant" if count == 1 else "descendants")
-        return "{title} ({kind}): {metadata}".format(title=self.title, kind=self.__class__.__name__, metadata=metadata)
+    kind = content_kinds.TOPIC
+    def derive_thumbnail(self):
+        from .files import TiledThumbnailFile
+        self.set_thumbnail(TiledThumbnailFile(self.get_non_topic_descendants()))
+        return self.thumbnail.get_filename()
 
     def validate(self):
         """ validate: Makes sure topic is valid
@@ -316,14 +384,82 @@ class TopicNode(ContentNode):
         """
         try:
             assert self.kind == content_kinds.TOPIC, "Assumption Failed: Node is supposed to be a topic"
-            assert self.questions == [], "Assumption Failed: Topic nodes should not have questions"
-            assert self.files == [], "Assumption Failed: Topic nodes should not have files"
-            assert self.extra_fields == {}, "Assumption Failed: Node should have empty extra_fields"
             return super(TopicNode, self).validate()
         except AssertionError as ae:
             raise InvalidNodeException("Invalid node ({}): {} - {}".format(ae.args[0], self.title, self.__dict__))
 
 
+class ContentNode(TreeNode):
+    """ Model representing the content nodes in the channel's tree
+
+        Base model for different content node kinds (topic, video, exercise, etc.)
+
+        Attributes:
+            source_id (str): content's original id
+            title (str): content's title
+            license (str or <License>): content's license
+            description (str): description of content (optional)
+            author (str): who created the content (optional)
+            thumbnail (str): local path or url to thumbnail image (optional)
+            files ([<File>]): list of file objects for node (optional)
+            extra_fields (dict): any additional data needed for node (optional)
+            domain_ns (str): who is providing the content (e.g. learningequality.org) (optional)
+    """
+    required_file_format = None
+
+    def __init__(self, source_id, title, license, license_description=None, copyright_holder=None, **kwargs):
+        self.set_license(license, copyright_holder=copyright_holder, description=license_description)
+        super(ContentNode, self).__init__(source_id, title, **kwargs)
+
+    def __str__(self):
+        metadata = "{0} {1}".format(len(self.files), "file" if len(self.files) == 1 else "files")
+        return "{title} ({kind}): {metadata}".format(title=self.title, kind=self.__class__.__name__, metadata=metadata)
+
+    def set_license(self, license, copyright_holder=None, description=None):
+        # Add license (create model if it's just a path)
+        if isinstance(license, str):
+            from .licenses import get_license
+            license = get_license(license, copyright_holder=copyright_holder, description=description)
+        self.license = license
+
+    def validate(self):
+        """ validate: Makes sure content node is valid
+            Args: None
+            Returns: boolean indicating if content node is valid
+        """
+        assert isinstance(self.license, str) or isinstance(self.license, License), "Assumption Failed: License is not a string or license object"
+        # if self.required_file_format:
+        #     files_valid = False
+        #     #not any(f for f in self.files if isinstance(f, DownloadFile))
+        #     for f in self.files:
+        #         files_valid = files_valid or (f.path.endswith(self.required_file_format)
+        #     assert files_valid , "Assumption Failed: Node should have at least one {} file".format(self.required_file_format)
+        return super(ContentNode, self).validate()
+
+    def to_dict(self):
+        """ to_dict: puts data in format CC expects
+            Args: None
+            Returns: dict of channel data
+        """
+        return {
+            "title": self.title,
+            "language" : self.language,
+            "description": self.description,
+            "node_id": self.get_node_id().hex,
+            "content_id": self.get_content_id().hex,
+            "source_domain": self.domain_ns.hex,
+            "source_id": self.source_id,
+            "author": self.author,
+            "files" : [f.to_dict() for f in filter(lambda x: x and x.filename, self.files)], # Filter out failed downloads
+            "kind": self.kind,
+            "license": self.license.license_id,
+            "license_description": self.license.description,
+            "copyright_holder": self.license.copyright_holder,
+            "questions": [question.to_dict() for question in self.questions],
+            "extra_fields": json.dumps(self.extra_fields),
+        }
+
+
 class VideoNode(ContentNode):
     """ Model representing videos in channel
 
@@ -332,47 +468,42 @@ class VideoNode(ContentNode):
         Attributes:
             source_id (str): content's original id
             title (str): content's title
+            license (str or <License>): content's license
             author (str): who created the content (optional)
             description (str): description of content (optional)
             derive_thumbnail (bool): indicates whether to derive thumbnail from video (optional)
-            license (str): content's license based on le_utils.constants.licenses
             thumbnail (str): local path or url to thumbnail image (optional)
-            copyright_holder (str): name of person or organization who owns license (optional)
             extra_fields (dict): any additional data needed for node (optional)
             domain_ns (str): who is providing the content (e.g. learningequality.org) (optional)
             files ([<File>]): list of file objects for node (optional)
     """
-    def __init__(self, source_id, title, derive_thumbnail=False, **kwargs):
-        self.kind = content_kinds.VIDEO
-        self.derive_thumbnail = derive_thumbnail
+    kind = content_kinds.VIDEO
+    required_file_format = file_formats.MP4
 
-        super(VideoNode, self).__init__(source_id, title, **kwargs)
-
-    def __str__(self):
-        metadata = "{0} {1}".format(len(self.files), "file" if len(self.files) == 1 else "files")
-        return "{title} ({kind}): {metadata}".format(title=self.title, kind=self.__class__.__name__, metadata=metadata)
+    def __init__(self, source_id, title, license, derive_thumbnail=False, **kwargs):
+        self.generate_thumbnail = derive_thumbnail
+        super(VideoNode, self).__init__(source_id, title, license, **kwargs)
 
     def process_files(self):
         """ download_files: Download video's files
             Args: None
             Returns: None
         """
-        from .files import VideoFile, ThumbnailFile, ExtractedVideoThumbnailFile
+        from .files import VideoFile, ExtractedVideoThumbnailFile, WebVideoFile
 
         downloaded = super(VideoNode, self).process_files()
 
         try:
             # Extract thumbnail if one hasn't been provided and derive_thumbnail is set
-            if self.derive_thumbnail and len(list(filter(lambda f: isinstance(f, ThumbnailFile), self.files))) == 0:
-                videos = list(filter(lambda f: isinstance(f, VideoFile), self.files))
-                assert len(videos) > 0 and videos[0].filename, "No videos downloaded for this node"
+            if self.generate_thumbnail and not self.has_thumbnail():
+                videos = [f for f in self.files if isinstance(f, VideoFile) or isinstance(f, WebVideoFile)]
+                assert len(videos) > 0 and videos[0].filename, "Cannot extract thumbnail (No videos found on node {0})".format(self.source_id)
 
-                thumbnail = ExtractedVideoThumbnailFile(config.get_storage_path(videos[0].filename))
-                self.add_file(thumbnail)
-                downloaded.append(thumbnail.process_file())
+                self.set_thumbnail(ExtractedVideoThumbnailFile(config.get_storage_path(videos[0].filename)))
+                downloaded.append(self.thumbnail.get_filename())
 
         except AssertionError as ae:
-            config.LOGGER.warning("\tWARNING: Cannot extract thumbnail ({0})".format(ae))
+            config.LOGGER.warning(ae)
 
         return downloaded
 
@@ -381,17 +512,14 @@ class VideoNode(ContentNode):
             Args: None
             Returns: boolean indicating if video is valid
         """
+        from .files import VideoFile, WebVideoFile
         try:
             assert self.kind == content_kinds.VIDEO, "Assumption Failed: Node should be a video"
-            assert self.license, "Assumption Failed: Video content must have a license"
             assert self.questions == [], "Assumption Failed: Video should not have questions"
             assert len(self.files) > 0, "Assumption Failed: Video must have at least one video file"
 
-            # Check if there are any .mp4 files
-            files_valid = False
-            for f in self.files:
-                files_valid = files_valid or file_formats.MP4 in f.path
-            assert files_valid , "Assumption Failed: Video should have at least one .mp4 file"
+            # Check if there are any .mp4 files if there are video files (other video types don't have paths)
+            assert any(f for f in self.files if isinstance(f, VideoFile) or isinstance(f, WebVideoFile)), "Assumption Failed: Video should have at least one .mp4 file"
 
             return super(VideoNode, self).validate()
         except AssertionError as ae:
@@ -406,40 +534,28 @@ class AudioNode(ContentNode):
         Attributes:
             source_id (str): content's original id
             title (str): content's title
+            license (str or <License>): content's license
             author (str): who created the content (optional)
             description (str): description of content (optional)
-            license (str): content's license based on le_utils.constants.licenses
             thumbnail (str): local path or url to thumbnail image (optional)
-            copyright_holder (str): name of person or organization who owns license (optional)
             extra_fields (dict): any additional data needed for node (optional)
             domain_ns (str): who is providing the content (e.g. learningequality.org) (optional)
             files ([<File>]): list of file objects for node (optional)
     """
-    def __init__(self, *args, **kwargs):
-        self.kind = content_kinds.AUDIO
-        super(AudioNode, self).__init__(*args, **kwargs)
-
-    def __str__(self):
-        metadata = "{0} {1}".format(len(self.files), "file" if len(self.files) == 1 else "files")
-        return "{title} ({kind}): {metadata}".format(title=self.title, kind=self.__class__.__name__, metadata=metadata)
+    kind = content_kinds.AUDIO
+    required_file_format = file_formats.MP3
 
     def validate(self):
         """ validate: Makes sure audio is valid
             Args: None
             Returns: boolean indicating if audio is valid
         """
+        from .files import AudioFile
         try:
             assert self.kind == content_kinds.AUDIO, "Assumption Failed: Node should be audio"
-            assert self.license, "Assumption Failed: Audio content must have a license"
             assert self.questions == [], "Assumption Failed: Audio should not have questions"
             assert len(self.files) > 0, "Assumption Failed: Audio should have at least one file"
-
-            # Check if there are any .mp3 or .wav files
-            files_valid = False
-            for f in self.files:
-                files_valid = files_valid or file_formats.MP3  in f.path
-            assert files_valid, "Assumption Failed: Audio should have at least one .mp3 file"
-
+            assert any(filter(lambda f: isinstance(f, AudioFile), self.files)), "Assumption Failed: Audio should have at least one audio file"
             return super(AudioNode, self).validate()
         except AssertionError as ae:
             raise InvalidNodeException("Invalid node ({}): {} - {}".format(ae.args[0], self.title, self.__dict__))
@@ -453,40 +569,28 @@ class DocumentNode(ContentNode):
         Attributes:
             source_id (str): content's original id
             title (str): content's title
+            license (str or <License>): content's license
             author (str): who created the content (optional)
             description (str): description of content (optional)
-            license (str): content's license based on le_utils.constants.licenses
             thumbnail (str): local path or url to thumbnail image (optional)
-            copyright_holder (str): name of person or organization who owns license (optional)
             extra_fields (dict): any additional data needed for node (optional)
             domain_ns (str): who is providing the content (e.g. learningequality.org) (optional)
             files ([<File>]): list of file objects for node (optional)
     """
-    def __init__(self, *args, **kwargs):
-        self.kind = content_kinds.DOCUMENT
-        super(DocumentNode, self).__init__(*args, **kwargs)
-
-    def __str__(self):
-        metadata = "{0} {1}".format(len(self.files), "file" if len(self.files) == 1 else "files")
-        return "{title} ({kind}): {metadata}".format(title=self.title, kind=self.__class__.__name__, metadata=metadata)
+    kind = content_kinds.DOCUMENT
+    required_file_format = file_formats.PDF
 
     def validate(self):
         """ validate: Makes sure document is valid
             Args: None
             Returns: boolean indicating if document is valid
         """
+        from .files import DocumentFile
         try:
             assert self.kind == content_kinds.DOCUMENT, "Assumption Failed: Node should be a document"
-            assert self.license, "Assumption Failed: Documents must have a license"
             assert self.questions == [], "Assumption Failed: Document should not have questions"
             assert len(self.files) > 0, "Assumption Failed: Document should have at least one file"
-
-            # Check if there are any .pdf files
-            files_valid = False
-            for f in self.files:
-                files_valid = files_valid or file_formats.PDF in f.path
-            assert files_valid, "Assumption Failed: Document should have at least one .pdf file"
-
+            assert any(filter(lambda f: isinstance(f, DocumentFile), self.files)), "Assumption Failed: Document should have at least one document file"
             return super(DocumentNode, self).validate()
         except AssertionError as ae:
             raise InvalidNodeException("Invalid node ({}): {} - {}".format(ae.args[0], self.title, self.__dict__))
@@ -501,40 +605,27 @@ class HTML5AppNode(ContentNode):
         Attributes:
             source_id (str): content's original id
             title (str): content's title
+            license (str or <License>): content's license
             author (str): who created the content (optional)
             description (str): description of content (optional)
-            license (str): content's license based on le_utils.constants.licenses
             thumbnail (str): local path or url to thumbnail image (optional)
-            copyright_holder (str): name of person or organization who owns license (optional)
             extra_fields (dict): any additional data needed for node (optional)
             domain_ns (str): who is providing the content (e.g. learningequality.org) (optional)
             files ([<File>]): list of file objects for node (optional)
     """
-    def __init__(self, *args, **kwargs):
-        self.kind = content_kinds.HTML5
-        super(HTML5AppNode, self).__init__(*args, **kwargs)
-
-    def __str__(self):
-        metadata = "{0} {1}".format(len(self.files), "file" if len(self.files) == 1 else "files")
-        return "{title} ({kind}): {metadata}".format(title=self.title, kind=self.__class__.__name__, metadata=metadata)
+    kind = content_kinds.HTML5
+    required_file_format = file_formats.HTML5
 
     def validate(self):
         """ validate: Makes sure HTML5 app is valid
             Args: None
             Returns: boolean indicating if HTML5 app is valid
         """
+        from .files import HTMLZipFile
         try:
             assert self.kind == content_kinds.HTML5, "Assumption Failed: Node should be an HTML5 app"
-            assert self.license, "Assumption Failed: HTML content must have a license"
             assert self.questions == [], "Assumption Failed: HTML should not have questions"
-
-            # Check if there are any .zip files
-            zip_file_found = False
-            for f in self.files:
-                if f.get_preset() == format_presets.HTML5_ZIP:
-                    zip_file_found = True
-            assert zip_file_found, "Assumption Failed: HTML does not have a .zip file attached"
-
+            assert any(filter(lambda f: isinstance(f, HTMLZipFile), self.files)), "Assumption Failed: HTML should have at least one html file"
             return super(HTML5AppNode, self).validate()
 
         except AssertionError as ae:
@@ -550,29 +641,32 @@ class ExerciseNode(ContentNode):
         Attributes:
             source_id (str): content's original id
             title (str): content's title
+            license (str or <License>): content's license
             author (str): who created the content (optional)
             description (str): description of content (optional)
-            license (str): content's license based on le_utils.constants.licenses (optional)
             exercise_data ({mastery_model:str, randomize:bool, m:int, n:int}): data on mastery requirements (optional)
             thumbnail (str): local path or url to thumbnail image (optional)
-            copyright_holder (str): name of person or organization who owns license (optional)
             extra_fields (dict): any additional data needed for node (optional)
             domain_ns (str): who is providing the content (e.g. learningequality.org) (optional)
             questions ([<Question>]): list of question objects for node (optional)
     """
-    default_preset = format_presets.EXERCISE
+    kind = content_kinds.EXERCISE
+
     def __init__(self, source_id, title, license, questions=None, exercise_data=None, **kwargs):
-        self.kind = content_kinds.EXERCISE
         self.questions = questions or []
 
         # Set mastery model defaults if none provided
-        exercise_data = {} if exercise_data is None else exercise_data
+        if not exercise_data:
+            exercise_data = {}
+        if isinstance(exercise_data, str):
+            exercise_data = {"mastery_model": exercise_data}
+
         exercise_data.update({
             'mastery_model': exercise_data.get('mastery_model') or exercises.M_OF_N,
             'randomize': exercise_data.get('randomize') or True,
         })
 
-        super(ExerciseNode, self).__init__(source_id, title, extra_fields=exercise_data, **kwargs)
+        super(ExerciseNode, self).__init__(source_id, title, license, extra_fields=exercise_data, **kwargs)
 
     def __str__(self):
         metadata = "{0} {1}".format(len(self.questions), "question" if len(self.questions) == 1 else "questions")
@@ -595,16 +689,38 @@ class ExerciseNode(ContentNode):
         for question in self.questions:
             downloaded += question.process_question()
 
-        # Update mastery model if parameters were not provided
-        if self.extra_fields['mastery_model'] == exercises.M_OF_N:
-            if 'n' not in self.extra_fields:
-                self.extra_fields.update({'n':self.extra_fields.get('m') or max(len(self.questions), 1)})
-            if 'm' not in self.extra_fields:
-                self.extra_fields.update({'m':self.extra_fields.get('n') or max(len(self.questions), 1)})
+        self.process_exercise_data()
 
         config.LOGGER.info("\t*** Images for {} have been processed".format(self.title))
         return downloaded
 
+    def process_exercise_data(self):
+        mastery_model = self.extra_fields['mastery_model']
+
+        # Keep original m/n values or other n/m values if specified
+        m_value = self.extra_fields.get('m') or self.extra_fields.get('n')
+        n_value = self.extra_fields.get('n') or self.extra_fields.get('m')
+
+        # Update mastery model if parameters were not provided
+        if mastery_model == exercises.M_OF_N:
+            m_value = m_value or max(min(5, len(self.questions)), 1)
+            n_value = n_value or max(min(5, len(self.questions)), 1)
+        elif mastery_model == exercises.DO_ALL:
+            m_value = n_value = max(len(self.questions), 1)
+        elif mastery_model == exercises.NUM_CORRECT_IN_A_ROW_10:
+            m_value = n_value = 10
+        elif mastery_model == exercises.NUM_CORRECT_IN_A_ROW_5:
+            m_value = n_value = 5
+        elif mastery_model == exercises.NUM_CORRECT_IN_A_ROW_3:
+            m_value = n_value = 3
+        elif mastery_model == exercises.NUM_CORRECT_IN_A_ROW_2:
+            m_value = n_value = 2
+        elif mastery_model == exercises.SKILL_CHECK:
+            m_value = n_value = 1
+
+        self.extra_fields.update({'m': m_value})
+        self.extra_fields.update({'n': n_value})
+
     def validate(self):
         """ validate: Makes sure exercise is valid
             Args: None
@@ -612,13 +728,17 @@ class ExerciseNode(ContentNode):
         """
         try:
             assert self.kind == content_kinds.EXERCISE, "Assumption Failed: Node should be an exercise"
-            assert "mastery_model" in self.extra_fields, "Assumption Failed: Exercise must have a mastery model in extra_fields"
 
             # Check if questions are correct
-            questions_valid = True
-            for q in self.questions:
-                questions_valid = questions_valid and q.validate()
-            assert questions_valid, "Assumption Failed: Exercise does not have a question"
+            assert any(self.questions), "Assumption Failed: Exercise does not have a question"
+            assert all(filter(lambda q: q.validate(), self.questions)), "Assumption Failed: Exercise has invalid question"
+            assert self.extra_fields['mastery_model'] in MASTERY_MODELS, "Assumption Failed: Unrecognized mastery model {}".format(self.extra_fields['mastery_model'])
             return super(ExerciseNode, self).validate()
         except AssertionError as ae:
             raise InvalidNodeException("Invalid node ({}): {} - {}".format(ae.args[0], self.title, self.__dict__))
+
+    def truncate_fields(self):
+        for q in self.questions:
+            q.truncate_fields()
+
+        super(ExerciseNode, self).truncate_fields()
