@@ -2,10 +2,12 @@
 
 import uuid
 import json
+import html
 import re
 import copy
 import sys
 from bs4 import BeautifulSoup
+from functools import partial
 from le_utils.constants import content_kinds,file_formats, format_presets, licenses, exercises
 from .. import config
 from ..exceptions import UnknownQuestionTypeError, InvalidQuestionException
@@ -31,7 +33,7 @@ class BaseQuestion:
             hints (str or [str]): optional hints on how to answer question
             raw_data (str): raw data for perseus file
     """
-    def __init__(self, id, question, question_type, answers=None, hints=None, raw_data=""):
+    def __init__(self, id, question, question_type, answers=None, hints=None, raw_data="", source_url=None, randomize=False):
         self.question = question
         self.question_type = question_type
         self.files=[]
@@ -39,8 +41,15 @@ class BaseQuestion:
         self.hints = [] if hints is None else [hints] if isinstance(hints,str) else hints
         self.raw_data = raw_data
         self.source_id=id
+        self.source_url = source_url
+        self.randomize = randomize
         self.id = uuid.uuid5(uuid.NAMESPACE_DNS, id)
 
+    def truncate_fields(self):
+        if self.source_url and len(self.source_url) > config.MAX_SOURCE_URL_LENGTH:
+            config.print_truncate("question_source_url", self.source_id, self.source_url)
+            self.source_url = self.source_url[:config.MAX_SOURCE_URL_LENGTH]
+
     def to_dict(self):
         """ to_dict: puts data in format CC expects
             Args: None
@@ -54,6 +63,8 @@ class BaseQuestion:
             "hints": json.dumps(self.hints, ensure_ascii=False),
             "answers": json.dumps(self.answers, ensure_ascii=False),
             "raw_data": self.raw_data,
+            "source_url": self.source_url,
+            "randomize": self.randomize,
         }
 
     def create_answer(self, answer, correct=True):
@@ -76,18 +87,22 @@ class BaseQuestion:
         # Process answers
         answers = []
         answer_files = []
+        answer_index = 0
         for answer in self.answers:
             processed_string, afiles = self.set_images(answer['answer'])
-            answers += [{"answer": processed_string, "correct": answer['correct']}]
+            answers.append({"answer": processed_string, "correct": answer['correct'], "order": answer_index})
+            answer_index += 1
             answer_files += afiles
         self.answers = answers
 
         # Process hints
         hints = []
         hint_files = []
+        hint_index = 0
         for hint in self.hints:
             processed_string, hfiles = self.set_images(hint)
-            hints += [{"hint": processed_string}]
+            hints.append({"hint": processed_string, "order": hint_index})
+            hint_index += 1
             hint_files += hfiles
         self.hints = hints
 
@@ -134,7 +149,7 @@ class BaseQuestion:
 
             alt_text = tag.get("alt") or ""
             tag.replaceWith("![{alt}]({src})".format(alt=alt_text, src=src_text))
-        return str(bs)
+        return html.unescape(bs.find('body').renderContents().decode('utf-8'))
 
     def set_image(self, text):
         """ set_image: Replace image string with downloaded image checksum
@@ -148,22 +163,25 @@ class BaseQuestion:
             return text, []
 
         # Set up return values and regex
-        graphie_reg = re.compile(WEB_GRAPHIE_URL_REGEX, flags=re.IGNORECASE)
-        graphie_match = graphie_reg.match(text)
         exercise_file = None
+        path_text = text.strip().replace('\\n', '')
+        graphie_reg = re.compile(WEB_GRAPHIE_URL_REGEX, flags=re.IGNORECASE)
+        graphie_match = graphie_reg.match(path_text)
         # If it is a web+graphie, download svg and json files,
         # Otherwise, download like other files
         if graphie_match:
-            text = graphie_match.group().replace("web+graphie:", "")
-            exercise_file = _ExerciseGraphieFile(text)
+            path_text = graphie_match.group().replace("web+graphie:", "")
+            exercise_file = _ExerciseGraphieFile(path_text)
         elif get_base64_encoding(text):
-            exercise_file = _ExerciseBase64ImageFile(text)
+            exercise_file = _ExerciseBase64ImageFile(path_text)
         else:
-            exercise_file = _ExerciseImageFile(text)
+            exercise_file = _ExerciseImageFile(path_text)
+
         exercise_file.assessment_item = self
         filename = exercise_file.process_file()
 
-        text = text.replace(text, exercises.CONTENT_STORAGE_FORMAT.format(exercise_file.get_replacement_str()))
+        text = text.replace(path_text, exercises.CONTENT_STORAGE_FORMAT.format(exercise_file.get_replacement_str()))
+
         return text, [exercise_file]
 
     def validate(self):
@@ -196,9 +214,9 @@ class PerseusQuestion(BaseQuestion):
             images ({key:str, ...}): a dict mapping image string to replace to path to image
     """
 
-    def __init__(self, id, raw_data):
+    def __init__(self, id, raw_data, source_url=None, **kwargs):
         raw_data = raw_data if isinstance(raw_data, str) else json.dumps(raw_data)
-        super(PerseusQuestion, self).__init__(id, "", exercises.PERSEUS_QUESTION, [], [], raw_data)
+        super(PerseusQuestion, self).__init__(id, "", exercises.PERSEUS_QUESTION, [], [], raw_data, source_url=source_url, **kwargs)
 
     def validate(self):
         """ validate: Makes sure perseus question is valid
@@ -219,9 +237,12 @@ class PerseusQuestion(BaseQuestion):
             Args: None
             Returns: list of all downloaded files
         """
-        image_files=[]
+        image_files = []
         image_data = json.loads(self.raw_data)
 
+        # process urls for widgets
+        self._recursive_url_find(image_data, image_files)
+
         # Process question
         if 'question' in image_data and 'images' in image_data['question']:
             image_data['question']['images'], qfiles = self.process_image_field(image_data['question'])
@@ -264,6 +285,32 @@ class PerseusQuestion(BaseQuestion):
             new_data[new_key] = new_data.pop(k)
         return new_data, files
 
+    def _recursive_url_find(self, item, image_list):
+        """ Utility function specifically for Khan Academy assessment items. Recursively iterates item
+            to find and replace all url image tags.
+            Args:
+                item (dict): KA assessment item
+                image_list (list): list of image file objects
+            Returns: None
+        """
+
+        recursive_fn = partial(self._recursive_url_find, image_list=image_list)
+
+        if isinstance(item, list):
+            list(map(recursive_fn, item))
+
+        elif isinstance(item, dict):
+            if 'url' in item:
+                if item['url']:
+                    item['url'], image_file = self.set_image(item['url'])
+                    image_list += image_file
+
+            for field, field_data in item.items():
+                if isinstance(field_data, dict):
+                    self._recursive_url_find(field_data, image_list)
+                elif isinstance(field_data, list):
+                    list(map(recursive_fn, field_data))
+
 
 class MultipleSelectQuestion(BaseQuestion):
     """ Model representing multiple select questions
@@ -282,9 +329,7 @@ class MultipleSelectQuestion(BaseQuestion):
             images ({key:str, ...}): a dict mapping image placeholder names to path to image
     """
 
-    def __init__(self, id, question, correct_answers, all_answers, hints=None):
-        hints = [] if hints is None else hints
-
+    def __init__(self, id, question, correct_answers, all_answers, **kwargs):
         # Put answers into standard format
         set_all_answers = set(all_answers)
         all_answers += [answer for answer in correct_answers if answer not in set_all_answers]
@@ -292,7 +337,7 @@ class MultipleSelectQuestion(BaseQuestion):
         if len(answers) == 0:
             answers = [self.create_answer('No answers provided.')]
             config.LOGGER.warning("\tWARNING: Question {id} does not have any answers (set to default)".format(id=id))
-        super(MultipleSelectQuestion, self).__init__(id, question, exercises.MULTIPLE_SELECTION, answers, hints)
+        super(MultipleSelectQuestion, self).__init__(id, question, exercises.MULTIPLE_SELECTION, answers, **kwargs)
 
     def validate(self):
         """ validate: Makes sure multiple selection question is valid
@@ -326,9 +371,7 @@ class SingleSelectQuestion(BaseQuestion):
             all_answers ([str]): list of all possible answers
             hint (str): optional hint on how to answer question
     """
-    def __init__(self, id, question, correct_answer, all_answers, hints=None):
-        hints = [] if hints is None else hints
-
+    def __init__(self, id, question, correct_answer, all_answers, **kwargs):
         # Put answers into standard format
         if correct_answer not in all_answers:
             all_answers += [correct_answer]
@@ -336,7 +379,7 @@ class SingleSelectQuestion(BaseQuestion):
         if len(answers) == 0:
             answers = [self.create_answer('No answers provided.')]
             config.LOGGER.warning("\tWARNING: Question {id} does not have any answers (set to default)".format(id=id))
-        super(SingleSelectQuestion, self).__init__(id, question, exercises.SINGLE_SELECTION, answers, hints)
+        super(SingleSelectQuestion, self).__init__(id, question, exercises.SINGLE_SELECTION, answers, **kwargs)
 
     def validate(self):
         """ validate: Makes sure single selection question is valid
@@ -359,37 +402,6 @@ class SingleSelectQuestion(BaseQuestion):
             raise InvalidQuestionException("Invalid question: {0}".format(self.__dict__))
 
 
-class FreeResponseQuestion(BaseQuestion):
-    """ Model representing free response questions
-
-        Free response questions are open-ended questions
-        that have no set answer (e.g. Prove that the sum of
-        every triangle's angles is 360 degrees.)
-
-        Attributes:
-            id (str): question's unique id
-            question (str): question text
-            hint (str): optional hint on how to answer question
-    """
-    def __init__(self, id, question, hints=None):
-        hints = [] if hints is None else hints
-        super(FreeResponseQuestion, self).__init__(id, question, exercises.FREE_RESPONSE, [], hints)
-
-    def validate(self):
-        """ validate: Makes sure free response question is valid
-            Args: None
-            Returns: boolean indicating if free response question is valid
-        """
-        try:
-            assert self.question_type == exercises.FREE_RESPONSE, "Assumption Failed: Question should be free response type"
-            for h in self.hints:
-                assert isinstance(h, str), "Assumption Failed: Hint in hint list is not a string"
-            assert self.answers == [], "Assumption Failed: Free response question should not have defined answers"
-            return super(FreeResponseQuestion, self).validate()
-        except AssertionError as ae:
-            raise InvalidQuestionException("Invalid question: {0}".format(self.__dict__))
-
-
 class InputQuestion(BaseQuestion):
     """ Model representing input questions
 
@@ -403,13 +415,12 @@ class InputQuestion(BaseQuestion):
             hint (str): optional hint on how to answer question
             images ({key:str, ...}): a dict mapping image placeholder names to path to image
     """
-    def __init__(self, id, question, answers, hints=None):
-        hints = [] if hints is None else hints
+    def __init__(self, id, question, answers, **kwargs):
         answers = [self.create_answer(answer) for answer in answers]
         if len(answers) == 0:
             answers = [self.create_answer('No answers provided.')]
             config.LOGGER.warning("\tWARNING: Question {id} does not have any answers (set to default)".format(id=id))
-        super(InputQuestion, self).__init__(id, question, exercises.INPUT_QUESTION, answers, hints)
+        super(InputQuestion, self).__init__(id, question, exercises.INPUT_QUESTION, answers, **kwargs)
 
     def validate(self):
         """ validate: Makes sure input question is valid
@@ -420,7 +431,11 @@ class InputQuestion(BaseQuestion):
             assert self.question_type == exercises.INPUT_QUESTION, "Assumption Failed: Question should be input answer type"
             assert len(self.answers) > 0, "Assumption Failed: Multiple selection question should have answers"
             for a in self.answers:
-                assert 'answer' in a and isinstance(a['answer'], str), "Assumption Failed: Answer in answer list is not a string"
+                assert 'answer' in a, "Assumption Failed: Answers must have an answer field"
+                try:
+                    float(a['answer'])
+                except ValueError:
+                    assert False, "Assumption Failed: Answer {} must be numeric".format(a['answer'])
             for h in self.hints:
                 assert isinstance(h, str), "Assumption Failed: Hint in hint list is not a string"
             return super(InputQuestion, self).validate()
