@@ -1,15 +1,15 @@
-import os
-import sys
-import requests
 import json
 import logging
-import webbrowser
-from . import config
-from .classes import nodes, questions
+import sys
+
+import requests
 from requests.exceptions import HTTPError
+import webbrowser
+
+from . import config, __version__
 from .managers.progress import RestoreManager, Status
 from .managers.tree import ChannelManager
-from importlib.machinery import SourceFileLoader
+from .sushi_bar_client import SushiBarClient
 
 # Fix to support Python 2.x.
 # http://stackoverflow.com/questions/954834/how-do-i-use-raw-input-in-python-3
@@ -18,51 +18,81 @@ try:
 except NameError:
     pass
 
-def uploadchannel(path, verbose=False, update=False, resume=False, reset=False, step=Status.LAST.name, token="#", prompt=False, publish=False, warnings=False, compress=False, **kwargs):
+__logging_handler = None
+
+
+def uploadchannel_wrapper(chef, args, options):
+    """
+    Call `uploadchannel` with SushiBar monitoring and progress reporting enabled.
+    Args:
+
+    """
+    try:
+        args_and_options = args.copy()
+        args_and_options.update(options)
+        uploadchannel(chef, **args_and_options)
+        config.SUSHI_BAR_CLIENT.report_stage('COMPLETED', 0)
+    except Exception as e:
+        if config.SUSHI_BAR_CLIENT:
+            config.SUSHI_BAR_CLIENT.report_stage('FAILURE', 0)
+        config.LOGGER.critical(str(e))
+        raise
+    finally:
+        if config.SUSHI_BAR_CLIENT:
+            config.SUSHI_BAR_CLIENT.close()
+        config.LOGGER.removeHandler(__logging_handler)
+
+
+def uploadchannel(chef, verbose=False, update=False, thumbnails=False, download_attempts=3, resume=False, reset=False, step=Status.LAST.name, token="#", prompt=False, publish=False, warnings=False, compress=False, stage=False, **kwargs):
     """ uploadchannel: Upload channel to Kolibri Studio server
         Args:
-            path (str): path to file containing construct_channel method
+            chef (BaseChef or subclass): class that implements the construct_channel method
             verbose (bool): indicates whether to print process (optional)
             update (bool): indicates whether to re-download files (optional)
+            thumbnails (bool): indicates whether to automatically derive thumbnails from content (optional)
+            download_attempts (int): number of times to retry downloading files (optional)
             resume (bool): indicates whether to resume last session automatically (optional)
             step (str): step to resume process from (optional)
             reset (bool): indicates whether to start session from beginning automatically (optional)
-            token (str): authorization token (optional)
+            token (str): content server authorization token
             prompt (bool): indicates whether to prompt user to open channel when done (optional)
             publish (bool): indicates whether to automatically publish channel (optional)
             warnings (bool): indicates whether to print out warnings (optional)
             compress (bool): indicates whether to compress larger files (optional)
-            kwargs (dict): keyword arguments to pass to sushi chef (optional)
+            stage (bool): indicates whether to stage rather than deploy channel (optional)
+            kwargs (dict): extra keyword args will be passed to get_channel and construct_channel (optional)
         Returns: (str) link to access newly created channel
     """
 
     # Set configuration settings
+    global __logging_handler
     level = logging.INFO if verbose else logging.WARNING if warnings else logging.ERROR
-    config.LOGGER.addHandler(logging.StreamHandler())
+    __logging_handler = logging.StreamHandler()
+    config.LOGGER.addHandler(__logging_handler)
     logging.getLogger("requests").setLevel(logging.WARNING)
     config.LOGGER.setLevel(level)
 
-    # Mount file:// to allow local path requests
-    config.SESSION.headers.update({"Authorization": "Token {0}".format(token)})
     config.UPDATE = update
     config.COMPRESS = compress
+    config.THUMBNAILS = thumbnails
+    config.STAGE = stage
+    config.PUBLISH = publish
 
-    # Authenticate user
-    if token != "#":
-        if os.path.isfile(token):
-            with open(token, 'r') as fobj:
-                config.SESSION.headers.update({"Authorization": "Token {0}".format(fobj.read())})
-        try:
-            response = config.SESSION.post(config.authentication_url())
-            response.raise_for_status()
-            user = json.loads(response._content.decode("utf-8"))
-            config.LOGGER.info("Logged in with username {0}".format(user['username']))
+    # Set max retries for downloading
+    config.DOWNLOAD_SESSION.mount('http://', requests.adapters.HTTPAdapter(max_retries=int(download_attempts)))
+    config.DOWNLOAD_SESSION.mount('https://', requests.adapters.HTTPAdapter(max_retries=int(download_attempts)))
 
-        except HTTPError:
-            config.LOGGER.error("Invalid token: Credentials not found")
-            sys.exit()
-    else:
-        prompt_token(config.DOMAIN)
+    # Get domain to upload to
+    config.init_file_mapping_store()
+
+    # Authenticate user and check current Ricecooker version
+    username, token = authenticate_user(token)
+    check_version_number()
+
+    # Set dashboard client settings
+    config.LOGGER.info("Running get_channel... ")
+    channel = chef.get_channel(**kwargs)
+    config.SUSHI_BAR_CLIENT = SushiBarClient(channel, username, token)
 
     config.LOGGER.info("\n\n***** Starting channel build process *****\n\n")
 
@@ -71,7 +101,7 @@ def uploadchannel(path, verbose=False, update=False, resume=False, reset=False,
     if (reset or not config.PROGRESS_MANAGER.check_for_session()) and step.upper() != Status.DONE.name:
         config.PROGRESS_MANAGER.init_session()
     else:
-        if resume or prompt_resume():
+        if resume or prompt_yes_or_no('Previous session detected. Would you like to resume your last session?'):
             config.LOGGER.info("Resuming your last session...")
             step = Status.LAST.name if step is None else step
             config.PROGRESS_MANAGER = config.PROGRESS_MANAGER.load_progress(step.upper())
@@ -80,7 +110,9 @@ def uploadchannel(path, verbose=False, update=False, resume=False, reset=False,
 
     # Construct channel if it hasn't been constructed already
     if config.PROGRESS_MANAGER.get_status_val() <= Status.CONSTRUCT_CHANNEL.value:
-        config.PROGRESS_MANAGER.set_channel(run_construct_channel(path, kwargs))
+        config.LOGGER.info("Calling construct_channel... ")
+        channel = chef.construct_channel(**kwargs)
+        config.PROGRESS_MANAGER.set_channel(channel)
     channel = config.PROGRESS_MANAGER.channel
 
     # Set initial tree if it hasn't been set already
@@ -90,6 +122,7 @@ def uploadchannel(path, verbose=False, update=False, resume=False, reset=False,
 
     # Download files if they haven't been downloaded already
     if config.PROGRESS_MANAGER.get_status_val() <= Status.DOWNLOAD_FILES.value:
+        print("Downloading files...")
         config.PROGRESS_MANAGER.set_files(*process_tree_files(tree))
 
     # Set download manager in case steps were skipped
@@ -98,6 +131,7 @@ def uploadchannel(path, verbose=False, update=False, resume=False, reset=False,
 
     # Get file diff if it hasn't been generated already
     if config.PROGRESS_MANAGER.get_status_val() <= Status.GET_FILE_DIFF.value:
+        print("Getting file diff...")
         config.PROGRESS_MANAGER.set_diff(get_file_diff(tree, files_to_diff))
     file_diff = config.PROGRESS_MANAGER.file_diff
 
@@ -106,71 +140,76 @@ def uploadchannel(path, verbose=False, update=False, resume=False, reset=False,
 
     # Upload files if they haven't been uploaded already
     if config.PROGRESS_MANAGER.get_status_val() <= Status.UPLOADING_FILES.value:
+        print("Uploading files...")
         config.PROGRESS_MANAGER.set_uploaded(upload_files(tree, file_diff))
 
     # Create channel on Kolibri Studio if it hasn't been created already
     if config.PROGRESS_MANAGER.get_status_val() <= Status.UPLOAD_CHANNEL.value:
+        print("Creating channel...")
         config.PROGRESS_MANAGER.set_channel_created(*create_tree(tree))
     channel_link = config.PROGRESS_MANAGER.channel_link
     channel_id = config.PROGRESS_MANAGER.channel_id
 
     # Publish tree if flag is set to True
-    if publish and config.PROGRESS_MANAGER.get_status_val() <= Status.PUBLISH_CHANNEL.value:
+    if config.PUBLISH and config.PROGRESS_MANAGER.get_status_val() <= Status.PUBLISH_CHANNEL.value:
+        print("Publishing channel...")
         publish_tree(tree, channel_id)
         config.PROGRESS_MANAGER.set_published()
 
     # Open link on web browser (if specified) and return new link
     config.LOGGER.info("\n\nDONE: Channel created at {0}\n".format(channel_link))
-    if prompt:
-        prompt_open(channel_link)
+    if prompt and prompt_yes_or_no('Would you like to open your channel now?'):
+        config.LOGGER.info("Opening channel... ")
+        webbrowser.open_new_tab(channel_link)
+
     config.PROGRESS_MANAGER.set_done()
     return channel_link
 
-def prompt_token(domain):
-    """ prompt_token: Prompt user to enter authentication token
-        Args: domain (str): domain to authenticate user
-        Returns: Authenticated response
+def authenticate_user(token):
     """
-    token = input("\nEnter authentication token ('q' to quit):").lower()
-    if token == 'q':
+    Add the content curation Authorizatino `token` header to `config.SESSION`.
+    """
+    config.SESSION.headers.update({"Authorization": "Token {0}".format(token)})
+    try:
+        response = config.SESSION.post(config.authentication_url())
+        response.raise_for_status()
+        user = json.loads(response._content.decode("utf-8"))
+        config.LOGGER.info("Logged in with username {0}".format(user['username']))
+        return user['username'], token
+    except HTTPError:
+        config.LOGGER.error("Invalid token: Credentials not found")
         sys.exit()
-    else:
-        try:
-            config.SESSION.headers.update({"Authorization": "Token {0}".format(token)})
-            response = config.SESSION.post(config.authentication_url())
-            response.raise_for_status()
-            return token
-        except HTTPError:
-            config.LOGGER.error("Invalid token. Please login to {0}/settings/tokens to retrieve your authorization token.".format(domain))
-            prompt_token(domain)
 
-def prompt_resume():
-    """ prompt_resume: Prompt user to resume last session if one exists
+def check_version_number():
+    response = config.SESSION.post(config.check_version_url(), data=json.dumps({"version": __version__}))
+    response.raise_for_status()
+    result = json.loads(response._content.decode('utf-8'))
+
+    if  result['status'] == 0:
+        config.LOGGER.info(result['message'])
+    elif result['status'] == 1:
+        config.LOGGER.warning(result['message'])
+    elif result['status'] == 2:
+        config.LOGGER.error(result['message'])
+        if not prompt_yes_or_no("Continue anyways?"):
+            sys.exit()
+    else:
+        config.LOGGER.error(result['message'])
+        sys.exit()
+
+def prompt_yes_or_no(message):
+    """ prompt_yes_or_no: Prompt user to reply with a y/n response
         Args: None
         Returns: None
     """
-    openNow = input("\nPrevious session detected. Would you like to resume your previous session? [y/n]:").lower()
-    if openNow.startswith("y"):
+    user_input = input("{} [y/n]:".format(message)).lower()
+    if user_input.startswith("y"):
         return True
-    elif openNow.startswith("n"):
+    elif user_input.startswith("n"):
         return False
     else:
-        return prompt_resume()
+        return prompt_yes_or_no(message)
 
-def run_construct_channel(path, kwargs):
-    """ run_construct_channel: Run sushi chef's construct_channel method
-        Args:
-            path (str): path to sushi chef file
-            kwargs (dict): additional keyword arguments
-        Returns: channel created from contruct_channel method
-    """
-    # Read in file to access create_channel method
-    mod = SourceFileLoader("mod", path).load_module()
-
-    # Create channel (using method from imported file)
-    config.LOGGER.info("Constructing channel... ")
-    channel = mod.construct_channel(**kwargs)
-    return channel
 
 def create_initial_tree(channel):
     """ create_initial_tree: Create initial tree structure
@@ -198,6 +237,7 @@ def process_tree_files(tree):
     # Fill in values necessary for next steps
     config.LOGGER.info("Processing content...")
     files_to_diff = tree.process_tree(tree.channel)
+    config.SUSHI_BAR_CLIENT.report_statistics(files_to_diff)
     tree.check_for_files_failed()
     return files_to_diff, config.FAILED_FILES
 
@@ -225,6 +265,7 @@ def upload_files(tree, file_diff):
     tree.reattempt_upload_fails()
     return file_diff
 
+
 def create_tree(tree):
     """ create_tree: Upload tree to Kolibri Studio
         Args:
@@ -234,23 +275,10 @@ def create_tree(tree):
     # Create tree
     config.LOGGER.info("\nCreating tree on Kolibri Studio...")
     channel_id, channel_link = tree.upload_tree()
+    # channel_id, channel_link = tree.upload_channel_structure()
 
     return channel_link, channel_id
 
-def prompt_open(channel_link):
-    """ prompt_open: Prompt user to open web browser
-        Args:
-            channel_link (str): url of uploaded channel
-        Returns: None
-    """
-    openNow = input("\nWould you like to open your channel now? [y/n]:").lower()
-    if openNow.startswith("y"):
-        config.LOGGER.info("Opening channel... ")
-        webbrowser.open_new_tab(channel_link)
-    elif openNow.startswith("n"):
-        return
-    else:
-        prompt_open(channel_link)
 
 def publish_tree(tree, channel_id):
     """ publish_tree: Publish tree to Kolibri
